###################################
###  A Linguagem R na Pesquisa  ###
###          1ª edição          ###
###      Filipe J. Zabala       ###
### www.estatisticaclassica.com ###
###   filipe.zabala@pucrs.br    ###
###   2019-01-08 e 2019-01-10   ###
###################################


# install.packages('matlib',dep=T)
library(matlib)
?echelon
?gaussianElimination
?solve

# exemplo 1 (solucao unica)
(A <- matrix(c(1,3,-2,5, 3,5,6,7, 2,4,3,8), nrow=3, byrow=T))
base::solve(A[,1:3],A[,4])
echelon(A)
echelon(A, reduced = F)
echelon(A, verbose = T)

# exemplo 2 (infinitas solucoes)
(B <- matrix(c(-2,1,1,4, -4,2,2,8, -6,3,3,12), nrow=3, byrow=T))
base::solve(B[,1:3],B[,4]) # so funciona para sistemas de solucao unica
echelon(B)
echelon(B, verbose = T)

# exemplo 3 (sem solucao)
(C <- matrix(c(1,-2,3,7, 2,3,-1,4, 4,1,3,1), nrow=3, byrow=T))
solve(C[,1:3],C[,4]) # so funciona para sistemas de solucao unica
echelon(C)
echelon(C, verbose = T)

# funcao que indica o tipo da solucao de um sistema (matriz r x (r+1))
escalona <- function(A, B, ...){
  library(matlib)
  mer <- echelon(A, B, ...)  # matriz escalonada reduzida
  
  if(!is_square_matrix(mer)){
    smc <- rowSums(abs(mer[,1:(ncol(sol)-1)]))  # soma modulo coeficientes
    ti <- mer[,ncol(mer)] # termo independente
    
    if(prod(smc)!=0) {
      msg <- 'Solucao unica'
      sol <- mer[,ncol(mer)]
      }
    else if(prod(smc+abs(ti))==0) {
      msg <- 'Infinitas solucoes'
      sol <- 'Em construcao'
      }
    else {
      msg <- 'Sem solucao'
      sol <- NA
      }
  }
  return(list(matriz.escalonada.reduzida = mer,
              tipo.solucao = msg,
              solucao = sol))
}

escalona(A, reduced = T)
escalona(B, verbose = T, fractions = T)
escalona(C, verbose = T, latex = T)



# testes
(sol <- escalona(B, reduced = T)$matriz.escalonada.reduzida)
(livre <- diag(sol) == 0)
(var_livre <- paste0('x', which(livre)))

parcela <- rep(NA, length = sum(livre))
k <- 0
for(i in which(!livre)){
  for(j in (i+1):(ncol(sol)-1)){
    k <- k+1
    parcela[k] <- paste0(-sol[i,j],'*',var_livre[j-1])
  }
  parcela <- c(parcela, sol[i,ncol(sol)])
}
parcela
paste(parcela, collapse = '+')
